const request = require('request');
const { Meta } = require('./constants');

/**
 * Core API Connector
 * @class ApiConnector
 */
class ApiConnector {
  /**
   * Creates an instance of ApiConnector.
   * @memberof ApiConnector
   */
  constructor() {
    this.user = '';
    this.uri = '';
    this.datapoint = {};
    this.apiKey = '';
    this.action = '';
  }

  /**
   * @param {String} user
   * @param {String} apiKey
   * @returns {ApiConnector}
   */
  from(user, apiKey) {
    this.user = user;
    this.apiKey = apiKey;
    return this;
  }

  /**
   * @param {Datapoint} datapoint
   * @returns {ApiConnector}
   */
  with(datapoint) {
    this.datapoint = datapoint;
    return this;
  }

  /**
   * @param {String} uri
   * @returns {ApiConnector}
   */
  to(uri) {
    this.uri = uri;
    return this;
  }

  /**
   * @param {String} action
   * @returns {ApiConnector}
   */
  post(action) {
    this.action = action;
    return this;
  }

  send() {
    this.datapoint.meta[Meta.USER] = this.user;
    this.datapoint.meta[Meta.API_KEY] = this.apiKey;

    const content = {
      [Meta.DATA]: JSON.stringify(this.datapoint),
      [Meta.ENCRYPTION]: false,
    };

    request({
      url: `${this.uri}/${this.action}`,
      method: this.getRequestType(this.action),
      json: content,
    }, (error, response, body) => { });
  }

  /**
   * @param {String} action
   * @returns {String}
   */
  getRequestType(action) {
    switch (action) {
      case ApiConnector.ACTION_AUTHORIZE:
        return 'POST';
      case ApiConnector.ACTION_LOG:
        return 'POST';
      default:
        return 'GET';
    }
  }
}

// API Connector properties
ApiConnector.ACTION_AUTHORIZE = 'authorize';
ApiConnector.ACTION_LOG = 'log';

module.exports = ApiConnector;
