const stackTrace = require('stack-trace');
const { Error, Vulnerability } = require('../core/compiled').proto;

module.exports = (exports, name) => {
  /**
   * The purpose of this module is for checking the compliance
   * of the modules required by the client code.
   *
   * Check vulnerable instantiation of Hashids module
   * @class HashidsPatch
   * @extends {exports}
   */
  class HashidsPatch extends exports {
    // new Hashids(salt)
    constructor() {
      const checkDefaultOrShortSalt = (args) => args &&
        (args.indexOf('salt' != -1) || // Contains the word salt, or
          args.length <= 16)           // is shorter than 16 characters

      const salt = arguments[0];

      if (checkDefaultOrShortSalt(salt)) {
        const moduleName = `module.${name}`;
        const message = `Contains default or short salt '${salt}'`;

        // Error
        const complianceError = new Error();
        complianceError.code = 1;
        complianceError.description = `Non-Compliance for ${moduleName}: ${message}`;
        complianceError.type = 'compliance';

        const trace = stackTrace.get()[1];
        if (trace) {
          complianceError.location = trace.getFileName();
          complianceError.line = trace.getLineNumber();
        }

        // Vulnerability
        const complianceVulnerability = new Vulnerability();
        complianceVulnerability.error = complianceError;
        complianceVulnerability.type = 'compliance';
        if (trace) {
          complianceVulnerability.location = trace.getFileName();
          complianceVulnerability.line = trace.getLineNumber();
        }

        bitSensor.apply((datapoint) => {
          datapoint.errors.push(complianceError);
          datapoint.vulnerabilities.push(complianceVulnerability);
        });
      }

      super(arguments);
    }
  }

  return HashidsPatch;
};
